use utf8;
use strict;
use Encode;
use Encode::JP::H2Z;
use Encode qw/decode decode_utf8/;
use Encode::Guess;
use Net::DNS;
use Nana::Mail;
use Nana::YukiWikiDB;

my $maildir="./maildb";
my $modifier_dbtype="Nana::YukiWikiDB";

my $adminmail='nanami@daiba.cx';
my $adminname='でんき予報速報';

%::database;

# メール一斉送信

sub mailallsend {
	my($subject,$body)=@_;
	my $mail;
	my %db;
	my @data;
	&open_db;
	foreach my $email(keys %::database) {
		@data=split(/\n/,$::database{$email});
		foreach(@data) {
			my ($name,$value)=split(/=/,$_);
			$db{$name}=$value;
		}
		if($db{"registed"} eq 1) {
			$mail=$db{mail};
			Nana::Mail::send(
				to=>$mail, to_name=> $mail . "様",
				from=>$adminmail, from_name=>$adminname,
				subject=>$subject,data=>$body);
		}
	}
	&close_db;
}

# カナ→ｶﾅ変換 for Encode.pm

sub z2h{
	my $string = shift;
	$string=~s/～/-/g;
	$string=~s/０/0/g;
	$string=~s/１/1/g;
	$string=~s/２/2/g;
	$string=~s/３/3/g;
	$string=~s/４/4/g;
	$string=~s/５/5/g;
	$string=~s/６/6/g;
	$string=~s/７/7/g;
	$string=~s/８/8/g;
	$string=~s/９/9/g;
	$string = Encode::encode("euc-jp", $string);
	Encode::JP::H2Z::z2h(\$string);
	return Encode::decode("euc-jp", $string);
}
# QUERY_STRINGエンコード
sub enc {
	my ($encoded) = shift;
	utf8::encode($encoded);
	$encoded =~ s/([^\w])/sprintf("%%%02X", unpack("C", $1))/eg;
	return "$encoded";
}

# $basehref, $basepath 取得 - これも面倒だからpyukiwikiから移植
sub getbasehref {
	# Thanks moriyoshi koizumi.
	my $basehost = "$ENV{'HTTP_HOST'}";
	$basehost = 'http://' . $basehost;
	# Special Thanks to gyo
	$basehost .= ":$ENV{'SERVER_PORT'}"
		if ($ENV{'SERVER_PORT'} ne '80' && $basehost !~ /:\d/);
	# URLの生成
	my $uri;
	my $req=$ENV{REQUEST_URI};
	$req=~s/\?.*//g;
	if($req ne '') {
		if($req eq $ENV{SCRIPT_NAME}) {
			$uri= $ENV{'SCRIPT_NAME'};
		} else {
			for(my $i=0; $i<length($ENV{SCRIPT_NAME}); $i++) {
				if(substr($ENV{SCRIPT_NAME},$i,1) eq substr($req,$i,1)) {
					$uri.=substr($ENV{SCRIPT_NAME},$i,1);
				} else {
					last;
				}
			}
		}
	} else {
		$uri .= $ENV{'SCRIPT_NAME'};
	}
	my $basepath=$uri;
	$basepath=~s/\/[^\/]*$//g;
	my $basehref=$basehost . $uri;
	return($basehref, $basehost, $basepath);
}

# gzip圧縮
sub gzip_compress {
	my $header=shift;
	my $gzip_command="gzip";
	my $gzip_path;
	my $gzip_header;
	my $fastflag;
	my $forceflag;
	foreach(split(/:/,$ENV{PATH})) {
		if(-x "$_/$gzip_command") {
			$gzip_path="$_/$gzip_command" ;
			if(open(PIPE,"$gzip_path --help 2>&1|")) {
				foreach(<PIPE>) {
					$forceflag="--force" if(/(\-\-force)/);
					$fastflag="--fast" if(/(\-\-fast)/);
				}
				close(PIPE);
			}
		}
	}
	$gzip_path="$gzip_path $fastflag $forceflag";
	if ($gzip_path ne '') {
		if(($ENV{'HTTP_ACCEPT_ENCODING'}=~/gzip/)) {
			if($ENV{'HTTP_ACCEPT_ENCODING'}=~/x-gzip/) {
				$gzip_header="Content-Encoding: x-gzip\n";
			} else {
				$gzip_header="Content-Encoding: gzip\n";
			}
		}
	}

	# サーバー出力
	print "$header\n";
	print "$gzip_header" if($gzip_header ne '');
	print "\n";

	if ($gzip_header ne '') {
		open(STDOUT,"| $gzip_path");
		binmode(STDOUT);
	}
}

# use モジュールと同等、なくても動作するようになっています。
sub load_module{
	my $mod = shift;
	eval qq( require $mod; );
	unless($@) {
		return 1;
	} else {
		return 0;
	}
}

# QR code module from Pyukiwiki by nanakochi123456
# Base:http://pyukiwiki.sourceforge.jp/PyukiWiki/Plugin/Nanami/qrcode/

# QRコードの画像自体を作成
sub make_qrcode {
	my ($string,$mode) = @_;
	my %hParm;
	my $oGdBar;

	my $defaultECC;
	my $defaultVersion;
	my $defaultSize;

	if($mode eq 'result') {
		$defaultECC='M';
		$defaultVersion=15;
		$defaultSize=3;
	} else {
		$defaultECC='M';
		$defaultVersion=6;
		$defaultSize=3;
	}

	if(&load_module("GD") && &load_module("GD::Barcode")) {
		$hParm{Ecc}=$defaultECC;
		$hParm{ModuleSize}=$defaultSize;
		$hParm{Version}=$defaultVersion;
		$oGdBar = GD::Barcode->new(
			'QRcode',
			$string,
			\%hParm);
		die($GD::Barcode::errStr) unless($oGdBar);

		binmode(STDOUT);
		print <<FIN;
Content-type: image/png

FIN
		print $oGdBar->plot->png;
		exit;
	}
}

sub force_decode($) {
	my $str = shift || '';
	my $enc = guess_encoding($str, qw/shiftjis utf8/);
	return ref $enc ? $enc->decode($str) : decode_utf8($str);
}

sub mailchk {
	my ($mail)=shift;
$::ismail=q((?:[^(\040)<>@,;:&#"'.\\\[\]\000-\037\x80-\xff](?:[^(\040)<>@,;:&#".\\\[\]\000-\037\x80-\xff])*(?![^(\040)<>@,;:&#".\\\[\]\000-\037\x80-\xff])|["'][^\\\x80-\xff\n\015"]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015"]*)*["'])(?:\.(?:[^(\040)<>@,;:&#"'.\\\[\]\000-\037\x80-\xff](?:[^(\040)<>@,;:&#".\\\[\]\000-\037\x80-\xff])*(?![^(\040)<>@,;:&#".\\\[\]\000-\037\x80-\xff])|["'][^\\\x80-\xff\n\015"]*(?:\\[^\x80-\xff][^\\\x80-\xff\n\015"]*)*["']))*\.?@(?:[^(\040)<>@,;:&#"'.\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:&#"'.\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\])(?:\.(?:[^(\040)<>@,;:&#"'.\\\[\]\000-\037\x80-\xff]+(?![^(\040)<>@,;:&#"'.\\\[\]\000-\037\x80-\xff])|\[(?:[^\\\x80-\xff\n\015\[\]]|\\[^\x80-\xff])*\]))*);
	return 0 if($mail!~/$::ismail/o);

	my($user,$domain)=split(/\@/,$mail);
	my $resolver = new Net::DNS::Resolver;
	$resolver->udp_timeout(30);
	my @mx = mx($resolver, $domain);
	if (@mx) {
		return 1;
	} else {
		return 0;
	}
} 

sub open_db {
	&dbopen($maildir,\%::database);
}

sub close_db {
	&dbclose(\%::database);
}

sub dbopen {
	my($dir,$db)=@_;
	tie(%$db, "$modifier_dbtype", $dir) or &print_error("(tie $modifier_dbtype) $dir");
}

sub dbclose {
	my($db)=@_;
	untie(%$db);
}

sub date {
	my ($format, $tm, $gmtime) = @_;
	my %weekday;
	my %ampm;

	# yday:0-365 $isdst Summertime:1/not:0
	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = 
		$gmtime ne '' && @_ > 2
			? ($tm+0 > 0 ? gmtime($tm) : gmtime(time))
			: ($tm+0 > 0 ? localtime($tm) : localtime(time));

	$year += 1900;
	my $hr12=$hour=>12 ? $hour-12:$hour;
	# year
	$format =~ s/Y/$year/ge;	# Y:4char ex)1999 or 2003

	# month
	$mon++;									# mon is 0 to 11 add 1
	$format =~ s/n/$mon/ge;					# n:1-12
	$mon = "0" . $mon if ($mon < 10);
	$format =~ s/m/$mon/ge;					# m:01-12

	# day
	$format =~ s/j/$mday/ge;				# j:1-31
	$mday = "0" . $mday if ($mday < 10);
	$format =~ s/d/$mday/ge;				# d:01-31

	# hour
	$hour = "0" . $hour if ($hour < 10);
	$format =~ s/H/$hour/ge;				# H:00-23

	# minutes
	$min = "0" . $min if ($min < 10);
	$format =~ s/i/$min/ge;					# i:00-59

	# second
	$sec = "0" . $sec if ($sec < 10);
	$format =~ s/s/$sec/ge;					# s:00-59

	return $format;
}

1;
